@@ -4,6 +4,9 @@
 
 (* Internal term representation *)
 
+open Elpi_util
+open Elpi_parser
+
 module Fmt = Format
 module F = Ast.Func
 open Util
@@ -506,7 +509,8 @@ module CHR : sig
     new_goal : sequent option;
     nargs : int [@default 0];
     pattern : constant list;
-    rule_name : string
+    rule_name : string;
+    rule_loc : Loc.t;
   }
   val pp_sequent : Fmt.formatter -> sequent -> unit
   val show_sequent : sequent -> string
@@ -537,6 +541,7 @@ end = struct (* {{{ *)
     nargs : int [@default 0];
     pattern : constant list;
     rule_name : string;
+    rule_loc : Loc.t;
   }
   [@@ deriving show]
   type t = {
@@ -644,10 +649,10 @@ let term_of_extra_goal = function
   | RawGoal x -> x
   | x ->
       Util.anomaly (Printf.sprintf "Unprocessed extra_goal: %s.\nOnly %s and %s can be left unprocessed,\nplease call API.RawData.set_extra_goals_postprocessing.\n"
-        (Obj.(extension_name (extension_constructor x)))
-        (Obj.(extension_name (extension_constructor (Unify(dummy,dummy)))))
-        (Obj.(extension_name (extension_constructor (RawGoal dummy)))))
-
+        (Obj.Extension_constructor.(name (of_val x)))
+        (Obj.Extension_constructor.(name (of_val (Unify(dummy,dummy)))))
+        (Obj.Extension_constructor.(name (of_val (RawGoal dummy)))))
+        
 end
 
 module ContextualConversion = struct
