@@ -25,3 +25,13 @@ let () = declare "findall"
   ~source_elpi:"findall.elpi"
   ~description:"stash across backtrack"
   ()
+
+let () =
+  let v = Sys.ocaml_version in
+  if Str.string_match (Str.regexp "4\\.\\(08\\|09\\|10\\|11\\|12\\)") v 0 then
+    () (* unix opem_process_* APIS are broken *)
+  else
+    declare "unix"
+      ~source_elpi:"unix.elpi"
+      ~description:"unix APIs"
+      ()
