@@ -290,6 +290,19 @@ let () = declare "IO_COLON"
   ~typecheck:true
   ()
 
+let () = declare "graft_replace_ok"
+  ~source_elpi:"graft_replace_ok.elpi"
+  ~description:"replacing a clase"
+  ~typecheck:true
+  ()
+
+let () = declare "graft_replace_err"
+  ~source_elpi:"graft_replace_err.elpi"
+  ~description:"replacing a clase"
+  ~typecheck:true
+  ~expectation:Test.(FailureOutput (Str.regexp "name attribute"))
+  ()
+
 let mk_tmp_file =
   let tmp = ref 0 in
   let dir = Filename.get_temp_dir_name () in
@@ -330,6 +343,39 @@ let () = declare "trace-browser2-elab"
   ~expectation:(SuccessOutputFile { sample; adjust = Util.strip_cwd; reference = "trace2.elab.json" })
   ()
 
+let sample = mk_tmp_file "trace3.json" ".new"
+let () = declare "trace-browser3"
+  ~source_elpi:"trace3.elpi"
+  ~description:"trace generation"
+  ~typecheck:false
+  ~trace:(On["json";"file://"^sample;"-trace-at";"0";"99";"-trace-only";"user"])
+  ~expectation:(SuccessOutputFile { sample; adjust = Util.strip_cwd; reference = "trace3.json" })
+  ()
+
+let sample = mk_tmp_file "trace3.elab.json" ".new"
+let () = declare "trace-browser3-elab"
+  ~source_json:"trace3.json"
+  ~description:"trace elaboration"
+  ~expectation:(SuccessOutputFile { sample; adjust = Util.strip_cwd; reference = "trace3.elab.json" })
+  ()
+
+let sample = mk_tmp_file "trace4.json" ".new"
+let () = declare "trace-browser4"
+  ~source_elpi:"trace4.elpi"
+  ~description:"trace generation"
+  ~typecheck:false
+  ~trace:(On["json";"file://"^sample;"-trace-at";"0";"99";"-trace-only";"user"])
+  ~expectation:(SuccessOutputFile { sample; adjust = Util.strip_cwd; reference = "trace4.json" })
+  ()
+
+let sample = mk_tmp_file "trace4.elab.json" ".new"
+let () = declare "trace-browser4-elab"
+  ~source_json:"trace4.json"
+  ~description:"trace elaboration"
+  ~expectation:(SuccessOutputFile { sample; adjust = Util.strip_cwd; reference = "trace4.elab.json" })
+  ()
+  
+
 let sample = mk_tmp_file "trace_chr.json" ".new"
 let () = declare "trace-browser-chr"
   ~source_elpi:"trace_chr.elpi"
