@@ -296,18 +296,25 @@ end
 
 let match_file ~log file adjust reference =
   let file = adjust file in
-  Util.write log (Printf.sprintf "Diffing %s against %s\n" file reference);
-  match
-    Util.exec ~timeout:5.0 ~env:(Unix.environment ())
-      ~log ~close_output:false
-      ~executable:"diff" ~args:["-u";"--strip-trailing-cr";reference;file] ()
-  with
-  | Util.Exit(0,_,_) -> true
-  | Util.Exit(n,_,_) ->
-    Util.write log (Printf.sprintf "Exit code: %d\n" n);
-    Util.write log (Printf.sprintf "Promotion: cp %s %s\n" file reference);
-    false
-  | _ -> false
+  let dig1 = Digest.to_hex (Digest.file file) in
+  let dig2 = Digest.(to_hex (file reference)) in
+  if dig1 = dig2 then begin
+    Util.write log (Printf.sprintf "Digest of %s and %s is %s\n" file reference dig2);
+    true
+  end else begin
+    Util.write log (Printf.sprintf "Diffing %s against %s\n" file reference);
+      match
+      Util.exec ~timeout:5.0 ~env:(Unix.environment ())
+        ~log ~close_output:false
+        ~executable:"diff" ~args:["-u";"--strip-trailing-cr";reference;file] ()
+    with
+    | Util.Exit(0,_,_) -> true
+    | Util.Exit(n,_,_) ->
+      Util.write log (Printf.sprintf "Exit code: %d\n" n);
+      Util.write log (Printf.sprintf "Promotion: cp %s %s\n" file reference);
+      false
+    | _ -> false
+  end
 
 
 module Elpi = struct
